<?php

namespace Drupal\webform_password_protection_handler\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\webform\Entity\Webform;
use Drupal\webform_password_protection_handler\Token;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Provides a custom form for password validation.
 */
class PasswordProtectionForm extends FormBase {

  /**
   * The token service.
   *
   * @var \Drupal\webform_password_protection_handler\Token
   */
  protected $token;

  /**
   * Constructs a new SiteSettingsForm.
   *
   * @param \Drupal\webform_password_protection_handler\Token $token
   *   The token service.
   */
  public function __construct(Token $token) {
    $this->token = $token;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('webform_password_protection_handler.JWT'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'password_protection_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $webform_id = NULL, Webform $webform = NULL) {
    $validated_form = FALSE;
    if (!$webform->getHandlers()->has('password_protection')) {
      $validated_form = TRUE;
    }
    else {
      $handle_instance = $webform->getHandler('password_protection');
      if ($handle_instance->getStatus() === FALSE) {
        $validated_form = TRUE;
      }
    }

    if ($validated_form) {
      $webform_id = $webform->id();
      $url = Url::fromRoute('entity.webform.canonical', ['webform' => $webform_id]);
      $response = new RedirectResponse($url->toString());
      $response->send();
    }

    $form_state->set('webform', $webform);

    $form['password'] = [
      '#type' => 'password',
      '#title' => $this->t('Enter password'),
      '#required' => TRUE,
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $webform = $form_state->get('webform');
    $password = $webform->getHandler('password_protection')->getSetting('password');
    if ($form_state->getValue('password') != $password) {
      $this->messenger()->addError($this->t('Incorrect password.'));
    }
    else {
      $webform_id = $webform->id();
      $key = sprintf('password_protection:%s:%s', $webform_id, $password);
      [, $access_token] = $this->token->encode(
        ['password' => $form_state->getValue('password')],
        $key, ['host' => $this->getRequest()->getHost()]
      );

      $url = Url::fromRoute(
        'entity.webform.canonical',
        ['webform' => $webform_id],
        [
          'query' => [
            'access_token' => $access_token,
          ],
        ]);

      $response = new RedirectResponse($url->toString());
      $response->send();
    }
  }

}
