<?php

namespace Drupal\webform_password_protection_handler\Plugin\WebformHandler;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Form submission handler.
 *
 * Limit a webform submission form.
 *
 * @WebformHandler(
 *   id = "password_protection",
 *   label = @Translation("Password Protection"),
 *   category = @Translation("Webform Handler"),
 *   description = @Translation("Check the password is correctly"),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class PasswordProtectionHandler extends WebformHandlerBase {

  use StringTranslationTrait;

  /**
   * The current Request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The token service.
   *
   * @var \Drupal\webform_password_protection_handler\Token
   */
  protected $tokenService;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->tokenService = $container->get('webform_password_protection_handler.JWT');
    $instance->request = $container->get('request_stack')->getCurrentRequest();

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['password_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Settings'),
    ];

    $form['password_settings']['password'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Password'),
      '#required' => TRUE,
      '#return_value' => TRUE,
      '#default_value' => $this->configuration['password'],
    ];

    return $this->setSettingsParents($form);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['password'] = $form_state->getValue([
      'password_settings',
      'password',
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function alterForm(array &$form, FormStateInterface $form_state, WebformSubmissionInterface $webform_submission) {
    $webform_id = $this->getWebform()->id();
    $access_token = $this->request->query->get('access_token');
    $key = sprintf('password_protection:%s:%s', $webform_id, $this->configuration['password']);
    $access_webform = FALSE;
    if ($access_token && ($token_data = $this->tokenService->decode($access_token, $key))) {
      if ($token_data['data']->password === $this->configuration['password']) {
        $access_webform = TRUE;
      }
    }

    if (!$access_webform) {
      $url = Url::fromRoute('webform_password_protection_handler.password_form', ['webform' => $this->getWebform()->id()]);
      $response = new RedirectResponse($url->toString());
      $response->send();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    return [];
  }

}
